package com.smartvault.proto;

/// EffectivePermissionFlags
/// 
/// These represent the permissions in NodeResponse.message.your_permissions.
/// These represent what a specific user can effectively do. This takes into
/// account several things, including:
/// 
/// A. Access Control List
/// B. License Type
/// C. Capabilities of Node (i.e. you can't create files under an account)
public final class EffectivePermissionFlags
{
    public static final int Write = 0x01;
    public static final int CreateFiles = 0x03;
    public static final int CreateFolders = 0x04;
    public static final int Copy = 0x08;
    public static final int Delete = 0x10;
    public static final int ManageSecurity = 0x20;
    public static final int Read = 0x40;
    public static final int CreateCompany = 0x1000;

    public static boolean canWrite(int perm)
    {
        return (perm & EffectivePermissionFlags.Write) == EffectivePermissionFlags.Write;                    
    }

    public static boolean canCreateFiles(int perm)
    {
        return (perm & EffectivePermissionFlags.CreateFiles) == EffectivePermissionFlags.CreateFiles;                    
    }

    public static boolean canCreateFolders(int perm)
    {
        return (perm & EffectivePermissionFlags.CreateFolders) == EffectivePermissionFlags.CreateFolders;                    
    }

    public static boolean canCopy(int perm)
    {
        return (perm & EffectivePermissionFlags.Copy) == EffectivePermissionFlags.Copy;                    
    }

    public static boolean canDelete(int perm)
    {
        return (perm & EffectivePermissionFlags.Delete) == EffectivePermissionFlags.Delete;
    }

    public static boolean canManageSecurity(int perm)
    {
        return (perm & EffectivePermissionFlags.ManageSecurity) == EffectivePermissionFlags.ManageSecurity;
    }

    public static  boolean canRead(int perm)
    {
        return (perm & EffectivePermissionFlags.Read) == EffectivePermissionFlags.Read;
    }

    public static boolean canCreateCompany(int perm)
    {
        return (perm & EffectivePermissionFlags.CreateCompany) == EffectivePermissionFlags.CreateCompany;
    }

    private static String AddPerm(boolean can, String bld, String name)
    {
        if (can)
        {
            if (bld.length() != 0)
            {
                bld = bld + ",";
            }
            bld = bld + name;
        }

        return bld;
    }

    public static String GetDisplayString(int perm)
    {
        String bld = "";
        bld = AddPerm(canWrite(perm), bld, "Write");
        bld = AddPerm(canCreateFiles(perm), bld, "Create Files");
        bld = AddPerm(canCreateFolders(perm), bld, "Create Folders");
        bld = AddPerm(canCopy(perm), bld, "Copy");
        bld = AddPerm(canDelete(perm), bld, "Delete");
        bld = AddPerm(canManageSecurity(perm), bld, "Manage Security");
        bld = AddPerm(canRead(perm), bld, "Read");
        bld = AddPerm(canCreateCompany(perm), bld, "Create Company");

        return "[" + bld + "]";
    }
};
