/*****************************************************************************
Copyright (c) 2011 SmartVault, Inc.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ******************************************************************************/
package com.smartvault.rest;

import com.smartvault.net.WebRequestInfo;
import com.smartvault.net.ClientNetContext;
import com.smartvault.proto.BasicUserInfoProto.BasicUserInfoResponse;
import com.smartvault.proto.SignUpNewUserProto.SignUpNewUserRequest;
import com.smartvault.proto.SignUpNewUserProto.SignUpNewUserResponse;
import com.smartvault.proto.DelegationTokenProto.DelegationTokenRequest;
import com.smartvault.proto.DelegationTokenProto.DelegationTokenResponse;
import com.smartvault.proto.SendEmailProto.SendEmailRequest;
import com.smartvault.proto.SendEmailProto.SendEmailResponse;

public class ClientProtocol {
	private ClientCfg cfg;

	public ClientProtocol(ClientCfg cfg) {
		this.cfg = cfg;
	}

	private ClientNetContext createClient(String noun, String verb)
			throws Exception {
		WebRequestInfo webRequestInfo = new WebRequestInfo(
				this.cfg.getRootUri(), this.cfg.getClientId(),
				this.cfg.getAccessToken());

		return ClientNetContext.open(webRequestInfo, noun, verb);
	}

	public BasicUserInfoResponse statUser(String userEmail) throws Exception {
		ClientNetContext ctx = createClient(Constants.STAT_USER_INFO_PATH + 
				Constants.SLASH
				+ userEmail, ClientNetContext.GET);
		return ctx.readPayloadBasicUserInfoResponse();
	}

	public SignUpNewUserResponse signUpNewUser(SignUpNewUserRequest request)
			throws Exception {
		ClientNetContext ctx = createClient(Constants.SIGN_UP_NEW_USER_PATH,
				ClientNetContext.POST);
		ctx.writePayload(request);
		return ctx.readPayloadSignUpNewUserResponse();
	}
	
    public SendEmailResponse requestEmail (String userEmail, SendEmailRequest rqst) throws Exception
    {
        ClientNetContext ctx = createClient(Constants.STAT_USER_INFO_PATH + "/" + userEmail, ClientNetContext.POST);
        ctx.writePayload(rqst);
        return ctx.readPayloadSendEmailResponse();
    }

	public UserCfg getDelegationAccessToken(String email) throws Exception {
		DelegationTokenRequest.Builder cbld = DelegationTokenRequest
				.newBuilder();
		cbld.setUserEmail(email);
		ClientNetContext ctx = createClient(Constants.DELEGATION_TOKEN_PATH,
				ClientNetContext.POST);
		ctx.writePayload(cbld.build());
		DelegationTokenResponse dtoken = ctx
				.readPayloadDelegationTokenResponse();
		return new UserCfg(email, dtoken.getMessage().getToken(), this.cfg);
	}

	public DelegatedProtocol createDelegatedProtocol(String email)
			throws Exception {
		return new DelegatedProtocol(getDelegationAccessToken(email));
	}
	
	
}